import os, signal, time, sys, shutil, subprocess
def evaluator_info(mode = 0):
	if mode:
		print "Noob evaluator UDV-delta0"
		print "Author: Eduard Nicodei"
	else:
		print r"""USAGE: eval.py [-OPTIONS] NAME
Evaluate the ONI/OJI-type aplication NAME

OPTIONS:
  NAME: You must supply this argument. It tells the evaluator what is the name of
         the application that you're evaluating. For example, if the source is
         foo.cpp, the .in and .out files are foo.in and foo.out, then the name is foo
                
  -s, --start           The number of the first test.
		                        
  -e, --end             The number of the last test. Keep in mind that there are cases
                          in which tests start from 1 and in not all cases the end number is 10.
                        
    -t, --timelimit     Defines the time limit (in seconds) for the application's execution.
                        
    -i, --inext         Extension to be used for the "in" file in the testing
                          Use it if the files end in, for example, ".IN"
                        
    -o, --okext         The same but for the "ok" file
                        
    -C, --comparer      (NOT IMPLEMENTED YET!)
                          Use another comparer, instead of the default one.
                        
    -x, --extratime     (NOT IMPLEMENTED YET!)
                          Set the extra-time allowed for the aplication to complete. Use it
                          in situations were the aplication is too slow and you want to see
                          if it gives the right answer
    
    -h, --help          Print this help message and exit
    -q                  Quiet mode (less output)
    -v                  Verbose mode (more output)
    -V                  Print the version and exit
"""

def sig_handler(called_sig, frame): #a custom signal handeler
    """Signal handler internal use only"""
    if called_sig == signal.SIGINT: #keyboard interrupt (CTRL-C) FIX NEEDED!
        try: childPID
        except NameError:
            pass
        else:
            os.kill(childPID, signal.SIGKILL)
            os.kill(subPID, signal.SIGKILL)
        print "CTRL-C detected. Exiting..."
        exit(1)

def run_app(application_name, time_limit):
    """Runs the aplication_name and kill it if it doesn't end
    within the time_limit
    
    The function returns a bool and a float:
        -> the status
            (True means error, False means everything is ok)
        -> the time taken
            (How much the aplication has runed)
    """
    global childPID, subPID
    
    #create a pipe for IPC (Inter Process Comunication)
    pipe_read, pipe_write = os.pipe();

    #get the current time. It will be used to determine for how long the aplication has been running
    beg_time = time.time()

    #fork
    childPID = os.fork()
    if childPID == 0: #code to be executed by the child

        #create a subprocess with the aplication, don't wait for it
        test_app = subprocess.Popen(['./' + application_name])

        #subPID = test_app.pid
        #write the pid of the application to the pipe
        #append "      0". The spaces are used to separest the PID from the rest.
        #the 0 is used to indicate the end of the spaces
        os.write(pipe_write, str(test_app.pid) + '      0')

        #wait for the aplication to complete
        os.wait()

        #write '1' to signal that the app has completed, close the pipe and exit
        os.write(pipe_write, '1')
        os.close(pipe_write)

        exit(0)
    subPID = 0

    #read the pid of the app. This is put in a while loop to help with synchronization
    while (subPID == 0): subPID = int(os.read(pipe_read, 6))

    #waiting loop: it waits until the time limit has been exeded or the application has finished.
    #In order to see when the aplication has ended (an '1' will be found in the pipe), we read character
    #if the character is 0, we append to the pipe another '0'. The reason is that if os.read doesn't read any
    #characters and pipe_read is open, it will hang, waiting for a character to read. By appending
    #'0'es we always give os.read characters.
    
    while time.time() - beg_time < time_limit:
        txt = os.read(pipe_read, 1)
        if txt == '0': os.write(pipe_write, '0')
        elif txt == '1': break
        
    #the time that the aplication has stoped (or was killed)
    end_time = time.time()

    #close the pipes
    os.close(pipe_read)
    os.close(pipe_write)
    
    #check if the time limit has been exeded     	
    if end_time - beg_time >= time_limit: #if so, kill the child and application and exit

        #kill 'em all

        os.kill(childPID, signal.SIGKILL)
        os.kill(subPID, signal.SIGKILL)
        
        #if you're paranoic, you can use the command below to call the system pidof. This will
        #check if the application is still running
	#os.system("proctokill=`pidof ./%s` && kill $proctokill" %application_name)
        os.wait()
        return True, end_time - beg_time
    else: #all went ok, the time limit hasn't been exeded
        os.wait()
        return False, end_time - beg_time

def copy_files(file1_path, file2_name, in_ext, ok_ext): #copies the .in and .ok files
    try: #exception handler
        shutil.copyfile(file1_path + in_ext, file2_name + '.in')
        shutil.copyfile(file1_path + ok_ext, file2_name + '.ok')
    except IOError: return True
    else: return False

def remove_files(file_name):    #removes the .in, .out and .ok files
    try: #as in copy_files(), this is an exception hanlder
        os.remove(file_name + '.in')
        os.remove(file_name.lower() + '.out')
        os.remove(file_name + '.ok')
    except OSError: return True
    else: return False

def arg_manage(arg): #argument manager *UD*!
	#0-status 1-name 2-start 3-end 4-time 5-.in 6-.ok 7-verbose
	values = [0, '', 0, 10, 1, '.in', '.ok', 0]
	arg_passed = ['You must supply the <name of the application> argument!\nUse --help to print available arguments\n',\
	'-s <start test or first test> argument not supplyed. Assuming that the first test is nr. 1\n',\
	'-e <end test or last test> argument not supplyed. Assuming that the last test is nr. 10\n',\
	'-t <time limit> argument not supplyed. Assuming that the time limit is one second\n',\
	'-in <in file extension> argument not suplyed. Assuming that the extension used for in files is .in\n',\
	'-ok <ok file extension> argument not suplyed. Assuming that the extension used for ok files is .ok\n']
	i = 1
	list_lenght = len(arg)
	while i < list_lenght:
        	if arg[i] == '-s' or arg[i] == '--start':
                        try: int(arg[i + 1])
                        except IndexError: print 'No value given for start test argument'
                        except ValueError:
                                print 'Invalid value for start test: ' + arg[i + 1]
                                values[0] = 1
                                return values, 0
                        else:
                            if arg[i + 1][0] != '-':
                                values[2] = int(arg[i + 1])
                                i += 1
                                arg_passed[1] = ''
                            else: print 'No value given for start test argument'
				
		elif arg[i] == '-e' or arg[i] == '--end':
                        try: int(arg[i + 1])
                        except IndexError: print 'No value given for end test argument'
                        except ValueError:
                                print 'Invalid value for end test: ' + arg[i + 1]
                                values[0] = 1
                                return values, 0
                        else:
                            if arg[i + 1][0] != '-':
                                values[3] = int(arg[i + 1])
                                i += 1
                                arg_passed[2] = ''
                            else: print 'No value given for end test argument'
				
		elif arg[i] == '-t' or arg[i] == '--timelimit':
                        try:float(arg[i + 1])
                        except IndexError: print 'No value given for timelimit argument'
                        except ValueError:
                                print 'Invalid value for timelimit: ' + arg[i + 1]
                                values[0] = 1
                                return values, 0
                        else:
                            if arg[i + 1][0] != '-':
                                values[4] = float(arg[i + 1])
                                i += 1
                                arg_passed[3] = ''
                            else: print 'No value given for timelimit argument'

		elif arg[i] == '-i' or arg[i] == '--inext':
                        try: arg[i + 1]
                        except IndexError: print 'No value given for in file extension'
                        else:
                            values[5] = arg[i + 1]
                            i += 1
                            arg_passed[4] = ''

		elif arg[i] == '-o' or arg[i] == '--okext':
                        try: arg[i + 1]
                        except IndexError: print 'No value given for ok file extension'
                        else:
                            values[6] = arg[i + 1]
                            i += 1
                            arg_passed[5] = ''
			
		elif arg[i] == '-q' or arg[i] == '--quiet':
			values[7] = -1
			
		elif arg[i] == '-v' or arg[i] == '--verbose':
			values[7] = 1
			
		elif arg[i] == '-V' or arg[i] == '--version':
			evaluator_info(1)
			values[0] = 2
			return values, 0
		
		elif arg[i] == '-h' or arg[i] == '--help':
			evaluator_info()
			values[0] = 2
			return values, 0
			
		elif arg[i] == '-C' or arg[i] == '--comparer':
			print "NIY: " + arg[i]
			i += 1
			
		elif arg[i] == '-x' or arg[i] == '--extratime':
			print "NIY: " + arg[i]
			i += 1
		
		elif arg[i][0] == '-':
			print 'Unknown argument: ' + arg[i]
			values[0] = 1
			return values, 0
			
		elif i == list_lenght - 1:
                        values[1] = arg[i]
                        arg_passed[0] = ''
                else:
			print 'Error in argument: ' + arg[i] + 'Missing "-" maybe?'
			values[0] = 1
			return values, 0
		i += 1
	return values, arg_passed
